@@ -10,7 +10,7 @@ use Cache::SizeAwareFileCache;
 use Tie::Restore;
 use Storable qw( freeze );
 
-$VERSION = '1.41.2';
+$VERSION = '1.41';
 
 # --------------------------------------------------------------------------
 
@@ -412,7 +412,7 @@ sub invalidate_cache_entry
 
 # --------------------------------------------------------------------------
 
-sub clear_cache
+sub clear
 {
   $CGI::Cache::THE_CACHE->clear();
 
